@@ -38,7 +38,8 @@ from IPython.core.error import StdinNotImplementedError
 from jupyter_client.session import Session
 from tornado import ioloop
 from tornado.queues import Queue, QueueEmpty
-from traitlets import (
+from traitlets.config.configurable import SingletonConfigurable
+from traitlets.traitlets import (
     Any,
     Bool,
     Dict,
@@ -51,7 +52,6 @@ from traitlets import (
     default,
     observe,
 )
-from traitlets.config.configurable import SingletonConfigurable
 from zmq.eventloop.zmqstream import ZMQStream
 
 from ipykernel.jsonutil import json_clean
@@ -76,6 +76,8 @@ class Kernel(SingletonConfigurable):
     # attribute to override with a GUI
     eventloop = Any(None)
 
+    processes: t.Dict[str, psutil.Process] = {}
+
     @observe("eventloop")
     def _update_eventloop(self, change):
         """schedule call to eventloop from IOLoop"""
@@ -95,8 +97,12 @@ class Kernel(SingletonConfigurable):
         """
     )
 
+    implementation: str
+    implementation_version: str
+    banner: str
+
     @default("shell_streams")
-    def _shell_streams_default(self):
+    def _shell_streams_default(self):  # pragma: no cover
         warnings.warn(
             "Kernel.shell_streams is deprecated in ipykernel 6.0. Use Kernel.shell_stream",
             DeprecationWarning,
@@ -108,7 +114,7 @@ class Kernel(SingletonConfigurable):
             return []
 
     @observe("shell_streams")
-    def _shell_streams_changed(self, change):
+    def _shell_streams_changed(self, change):  # pragma: no cover
         warnings.warn(
             "Kernel.shell_streams is deprecated in ipykernel 6.0. Use Kernel.shell_stream",
             DeprecationWarning,
@@ -131,7 +137,7 @@ class Kernel(SingletonConfigurable):
     iopub_socket = Any()
     iopub_thread = Any()
     stdin_socket = Any()
-    log = Instance(logging.Logger, allow_none=True)
+    log: logging.Logger = Instance(logging.Logger, allow_none=True)  # type:ignore[assignment]
 
     # identities:
     int_id = Integer(-1)
@@ -263,7 +269,7 @@ class Kernel(SingletonConfigurable):
         for msg_type in self.control_msg_types:
             self.control_handlers[msg_type] = getattr(self, msg_type)
 
-        self.control_queue: Queue[Any] = Queue()
+        self.control_queue: Queue[t.Any] = Queue()
 
     def dispatch_control(self, msg):
         self.control_queue.put_nowait(msg)
@@ -531,7 +537,7 @@ class Kernel(SingletonConfigurable):
     def start(self):
         """register dispatchers for streams"""
         self.io_loop = ioloop.IOLoop.current()
-        self.msg_queue: Queue[Any] = Queue()
+        self.msg_queue: Queue[t.Any] = Queue()
         self.io_loop.add_callback(self.dispatch_queue)
 
         self.control_stream.on_recv(self.dispatch_control, copy=False)
@@ -677,7 +683,6 @@ class Kernel(SingletonConfigurable):
 
     async def execute_request(self, stream, ident, parent):
         """handle an execute_request"""
-
         try:
             content = parent["content"]
             code = content["code"]
@@ -941,7 +946,6 @@ class Kernel(SingletonConfigurable):
 
     async def debug_request(self, stream, ident, parent):
         content = parent["content"]
-
         reply_content = self.do_debug_request(content)
         if inspect.isawaitable(reply_content):
             reply_content = await reply_content
@@ -949,10 +953,8 @@ class Kernel(SingletonConfigurable):
         reply_msg = self.session.send(stream, "debug_reply", reply_content, parent, ident)
         self.log.debug("%s", reply_msg)
 
-    # Taken from https://github.com/jupyter-server/jupyter-resource-usage/blob/e6ec53fa69fdb6de8e878974bcff006310658408/jupyter_resource_usage/metrics.py#L16
     def get_process_metric_value(self, process, name, attribute=None):
         try:
-            # psutil.Process methods will either return...
             metric_value = getattr(process, name)()
             if attribute is not None:  # ... a named tuple
                 return getattr(metric_value, attribute)
@@ -967,18 +969,30 @@ class Kernel(SingletonConfigurable):
         reply_content = {"hostname": socket.gethostname(), "pid": os.getpid()}
         current_process = psutil.Process()
         all_processes = [current_process] + current_process.children(recursive=True)
-        process_metric_value = self.get_process_metric_value
+        # Ensure 1) self.processes is updated to only current subprocesses
+        # and 2) we reuse processes when possible (needed for accurate CPU)
+        self.processes = {
+            process.pid: self.processes.get(process.pid, process) for process in all_processes
+        }
         reply_content["kernel_cpu"] = sum(
-            [process_metric_value(process, "cpu_percent", None) for process in all_processes]
+            [
+                self.get_process_metric_value(process, "cpu_percent", None)
+                for process in self.processes.values()
+            ]
         )
+        mem_info_type = "pss" if hasattr(current_process.memory_full_info(), "pss") else "rss"
         reply_content["kernel_memory"] = sum(
-            [process_metric_value(process, "memory_info", "rss") for process in all_processes]
+            [
+                self.get_process_metric_value(process, "memory_full_info", mem_info_type)
+                for process in self.processes.values()
+            ]
         )
         cpu_percent = psutil.cpu_percent()
         # https://psutil.readthedocs.io/en/latest/index.html?highlight=cpu#psutil.cpu_percent
         # The first time cpu_percent is called it will return a meaningless 0.0 value which you are supposed to ignore.
         if cpu_percent is not None and cpu_percent != 0.0:
             reply_content["host_cpu_percent"] = cpu_percent
+        reply_content["cpu_count"] = psutil.cpu_count(logical=True)
         reply_content["host_virtual_memory"] = dict(psutil.virtual_memory()._asdict())
         reply_msg = self.session.send(stream, "usage_reply", reply_content, parent, ident)
         self.log.debug("%s", reply_msg)
@@ -990,7 +1004,7 @@ class Kernel(SingletonConfigurable):
     # Engine methods (DEPRECATED)
     # ---------------------------------------------------------------------------
 
-    async def apply_request(self, stream, ident, parent):
+    async def apply_request(self, stream, ident, parent):  # pragma: no cover
         self.log.warning("apply_request is deprecated in kernel_base, moving to ipyparallel.")
         try:
             content = parent["content"]
@@ -1028,7 +1042,7 @@ class Kernel(SingletonConfigurable):
     # Control messages (DEPRECATED)
     # ---------------------------------------------------------------------------
 
-    async def abort_request(self, stream, ident, parent):
+    async def abort_request(self, stream, ident, parent):  # pragma: no cover
         """abort a specific msg by id"""
         self.log.warning(
             "abort_request is deprecated in kernel_base. It is only part of IPython parallel"
@@ -1047,7 +1061,7 @@ class Kernel(SingletonConfigurable):
         )
         self.log.debug("%s", reply_msg)
 
-    async def clear_request(self, stream, idents, parent):
+    async def clear_request(self, stream, idents, parent):  # pragma: no cover
         """Clear our namespace."""
         self.log.warning(
             "clear_request is deprecated in kernel_base. It is only part of IPython parallel"
@@ -1067,7 +1081,7 @@ class Kernel(SingletonConfigurable):
         """prefixed topic for IOPub messages"""
         base = "kernel.%s" % self.ident
 
-        return ("%s.%s" % (base, topic)).encode()
+        return (f"{base}.{topic}").encode()
 
     _aborting = Bool(False)
 
@@ -1193,9 +1207,7 @@ class Kernel(SingletonConfigurable):
                 # zmq.select() is also uninterruptible, but at least this
                 # way reads get noticed immediately and KeyboardInterrupts
                 # get noticed fairly quickly by human response time standards.
-                rlist, _, xlist = zmq.select(
-                    [self.stdin_socket], [], [self.stdin_socket], 0.01  # type:ignore[arg-type]
-                )
+                rlist, _, xlist = zmq.select([self.stdin_socket], [], [self.stdin_socket], 0.01)
                 if rlist or xlist:
                     ident, reply = self.session.recv(self.stdin_socket)
                     if (ident, reply) != (None, None):
