@@ -10,6 +10,7 @@ import time
 from contextlib import contextmanager
 from subprocess import PIPE, Popen
 
+import pytest
 from flaky import flaky
 from jupyter_client import BlockingKernelClient
 from jupyter_core import paths
@@ -18,6 +19,10 @@ SETUP_TIMEOUT = 60
 TIMEOUT = 15
 
 
+if os.name == "nt":
+    pytest.skip("skipping tests on windows", allow_module_level=True)
+
+
 @contextmanager
 def setup_kernel(cmd):
     """start an embedded kernel in a subprocess, and wait for it to be ready
